#! /usr/bin/perl

use Getopt::Std;
use Bio::Perl;
use Bio::SimpleAlign;
use Bio::AlignIO;
use Bio::LocatableSeq;
use Bio::Location::Simple;
use Bio::Seq;
use Data::Dumper;
my %opts;
getopts('T:S:N:O:r:m:U:P:R:s:a:w:',\%opts);

my $SSU;
my $LSU;
my $tarAlign; #enthaelt die target-Alignments
my $stemAlign;

$opts{S} =~ /([^\/]+\/)*([^\.]+)\./; #alles was nicht Pfad und nicht Dateiendung ist, im Idealfall ist das der Name der snoRNA
my $sno = $2;
my $dir = $sno.".out";  #name of output-directory
my $out;


if(defined($opts{T}) && defined($opts{S}) && defined($opts{O})){
  if(defined( $opts{w})){
    $dir=$opts{w};
    }
  print "########".$sno."########\n";
  my $str;
  #    foreach(@targetRNAs){
  #	my $target = $_;
  #	$target =~ s/\.aln//g;
  #	print $target."\t";
  #	$str = 	Bio::AlignIO -> new (-file => "$opts{T}/".$target.".aln", -format => 'clustalw'); #einlesen eines TargetAlignment *.aln
  #	$targetAligns{$target} = $str -> next_aln();
  #    }
  #    print "\n";
  #    exit(0);
  $str = Bio::AlignIO -> new (-file => "$opts{T}", -format => 'clustalw');	#einlesen des TarAlignments
  $tarAlign = $str -> next_aln();
  $str = Bio::AlignIO -> new (-file => "$opts{S}", -format => 'clustalw');	#einlesen des StemAlignments
  $stemAlign = $str -> next_aln();
  
  
  if(!defined($opts{O})){ $opts{O} = "homo";}
  if(!defined($opts{N})){ $opts{N} = 2;}
  if(!defined($opts{R})){ $opts{R} = 1;}
  if(!defined($opts{e})){ $opts{e} = 200;}	 
  if(!defined($opts{j})){ $opts{j} = 11;}
  if(!defined($opts{k})){ $opts{k} = 18;}
  if(!defined($opts{r})){ $opts{r} = "yeast.svm.range";}
  if(defined($opts{P}) || defined($opts{U})){	
    if(!defined($opts{m})){$opts{m} = "yeast.svm.scale.12.model";}	
    if(!defined($opts{s})){$opts{s} = "u1_to_30.out";}	
  }
  else{
    if(!defined($opts{m})){$opts{m} = "yeast.svm.scale.5.model";}
  }	
  #	if(!defined($opts{k})){ $opts{k} = 16;}	
  #	if(!defined($opts{e})){ $opts{e} = 30;}
  #	if(!defined($opts{o})){ $opts{o} = -330;}	
  #	if(!defined($opts{f})){ $opts{f} = 1;}	
  #	if(!defined($opts{v})){ $opts{v} = 5;}	
  #	if(!defined($opts{w})){ $opts{w} = 5;}	
  #	if(!defined($opts{L})){ $opts{L} = 20;}	
  #	if(!defined($opts{x})){ $opts{x} = -1370;}	
  
  compute();
}
else{
  usage()
}

sub compute{
  `mkdir $dir`;
  open(PRED,">>$dir/$dir.target");
  my $best = selsnoop(); #first RNAsnoop-step, single sequence with specified organism
  cutAlign($best); #a second RNASnoop-step with alignments around in first step gained candidate regions
  close(PRED);
  open(TAB,">>$dir/predictions");
  printf TAB "%-15s\t%-15s\t%-10s\t%-8s\t%-20s\t%-105s\t%-45s\t%-85s\t%-12s\t%-18s\t%-6s\t%-7s\n","snoRNA","svm-target","svm-score","energy","TargetSequence","structure","file","figure","alignmentPos","Seq in Alignment","energy","figures";
  mktable($out);
  close(TAB);
  `mkdir $dir/ps_files`;
  `mv *aln_*_*.ps $dir/ps_files`;
  `mv $dir/*.ps $dir/ps_files`;
}
##############################################################
sub selsnoop
  #performs single sequence RNAsnoop for reference sequence
  ##############################################################
  {
    #hier mach ich ne Vorauswahl der Hits durch snoop erst nur in gewaehltem Organismus
    my $best;
    foreach my $seq($stemAlign->each_seq()){	#extrahieren der Stemsequenz des gewaehlten Organismus
      if($seq->display_name =~ /$opts{O}/){
	my $stem = $seq -> seq();
	$stem =~ s/-//g;			
	printfile("$dir/tmpStem", ">".$seq->display_name()."\n".$stem);	#schreiben der snoRNAFile fuer snoop
      }
    }
    my $preSel=();
    my $cnt = 0;
    foreach my $seq($tarAlign->each_seq()){		#extrahieren der Targetsequenz des preSelOrganismus
      if($seq->display_name() =~ /$opts{O}/){
	my $tar = $seq -> seq();
	$tar =~ s/-//g;			#ohne gaps
	if($cnt == 0){
	  printfile("$dir/tmpTarg", ">".$opts{O}."\n".$tar."\n");	#schreiben der TargetFile fuer snoop 
	}
	else{
	  printfile("$dir/tmpTarg", ">".$opts{O}."\n".$tar."\n");	#schreiben der TargetFile fuer snoop 
	}
	$cnt++;
      }
    }
    
    
    print "computing SVM-predictions for ".$opts{O}."...\n";
    
    if(defined $opts{U}){
      $preSel .= qx(apply_svm.pl -t $dir/tmpTarg -s $dir/tmpStem -U $opts{U} -S $opts{s} -r $opts{r} -m $opts{m} -o $dir);
    }
    elsif(defined $opts{P}){
      $preSel .= qx(apply_svm.pl -t $dir/tmpTarg -s $dir/tmpStem -P $opts{P} -r $opts{r} -m $opts{m} -o $dir);
    }
    else{
      $preSel .= qx(apply_svm.pl -t $dir/tmpTarg -s $dir/tmpStem -r $opts{r} -m $opts{m} -o $dir);
    }
    
    if(length($preSel)>1){
      my $results = parseSnoop($preSel);
      print PRED "###############SINGLE#########################\n";
      print "done\n";#.$preSel."\n";	
      $best = bestSel($results);
    }
    else{print "kein Target gefunden\n";exit(0);}
    return $best;
  }
  ##################################################################
    sub parseSnoop
    #parses RNAsnoop output
    ##################################################################
    {
    my $preSel = shift; 
    my $snoop;
    my $results;
    my $snoRNA;
    my $rRNA;
    my $score;
    my $header;
    my $key;
    @{$snoop} = split(/\n/,$preSel);  #die snoop ausgabe zeilenweise im Array 
    for(my $i = 0; $i < $#$snoop; $i += 2){		#durchlaufen der hits die snoop zurueckgibt  
	my $hit;
	#print $snoop->[$i].$snoop->[$i+1];
	if($snoop->[$i] =~ /^>/ && $snoop->[$i+1] =~ /^>/){
	    $header = $snoop->[$i];
	    $header =~/^>1.(0.\d+)(\w+)/ ; #header den svm-apply erzeugt, enthaelt svm-score
	    $header .= "_".$snoop->[$i+1];
	    $score = $1;
	    $header =~ s/>//g;
#	    print $score."\n";
	    $snoRNA = $sno;
	    $rRNA=$snoop->[$i+1];
	    $rRNA =~ s/>//;
#	    if($rRNA=~/18S/){$tarAlign= $SSU;}
#	    else{$tarAlign=$LSU;}
	    $key = $rRNA; #name of the targetRNA, followed by an inserted '_preSelOrg'
	    my $rep = "_".$opts{O};
	    $key =~ s/$rep//;
#	    print "TEST".$snoRNA."\t".$rRNA." = ".$key."\t".$score."\n";
	}
	$hit->{snoRNA}=$snoRNA;
	$hit->{rRNA}= $rRNA;
	$hit->{header}= $header;
	$hit->{score}=$score;
	$hit->{key}= $key;
	if($snoop->[$i] =~ /^[<.]/){
	    $hit->{rawStruc} = $snoop->[$i];
	    $hit->{rawSeq} = $snoop->[$i+1];
	    $hit->{rawSeq} =~ /^([^&]+)\&/;
	    $hit->{tarSeq} = $1;
	    $snoop->[$i] =~ /^([^&]+)\&([^\d]+)(\d+),(\d+)\s*;\s*(\d+)\s*:\s*(\d+)\s*,\s*(\d+)\s*\(([^=]+)\s*=/;  #Ausgabe von snoop
	    $hit->{tarStruc} = $1;
	    $hit->{stemStruc} = $2;
	    $hit->{struc} = $1."&".$2;
	    $hit->{start} = $3;
	    $hit->{end} = $4;
	    $hit->{u} = $5;
	    $hit->{energy} = $8;
	    if(defined($opts{a})){$hit->{score}=$hit->{energy};} #wenn -a gesetzt ist die energy der score
	    $hit->{Tlength} =  $hit->{end}-$hit->{start} + 1; #Laenge der Bindung im preSelOrg
	    $hit->{Tleft} =  $hit->{u}-$hit->{start};	#da steckt die Position des Us im target drin
	    $hit->{Tright} =  $hit->{end}-$hit->{u};
	    $hit->{uAlign} = findAlignPos($tarAlign,$hit->{u},$rRNA);
	    $hit->{startAlign} = findAlignPos($tarAlign,$hit->{start},$rRNA)-5;

	    if($hit->{startAlign}<0){$hit->{startAlign}=1;}

	    $hit->{endAlign} = findAlignPos($tarAlign,$hit->{end},$rRNA)+5;

	 #   print $hit->{start}."\t".$hit->{u}."\t".$hit->{end}."\n";
	 #   print $hit->{startAlign}."\t".$hit->{uAlign}."\t".$hit->{endAlign}."\n";
#TODO	    if($hit->{endAlign}>ENDE ALIGNMENT){$hit->{start}=ENDE ALIGNMENT;}
	    $hit->{aliLen} = $hit->{endAlign} - $hit->{startAlign} +1;
	    $hit->{Talign} = $tarAlign -> slice($hit->{startAlign},$hit->{endAlign});

	    if($hit->{Talign}->no_sequences >= $opts{N}){  #wenn genug Sequenzen im Alignment
		push @{$results},$hit;
	    }
	}
    } 
    return $results;
}
##############################################################################
sub bestSel
#sorts by score (energy or svm) and gives back r best results
##############################################################################
{
    my $unsorted = shift;
    my $best;
    if(defined($unsorted) && scalar(@{$unsorted}) > 0){
	my @sorted = sort {$a->{score} <=> $b->{score}} @{$unsorted}; #nach Energien absteigend geordnet
	my $table; #zum Entfernen von Duplikaten
	my $run = $opts{R};  #die r besten
	if(scalar(@sorted)<$opts{R}){
	    $run = scalar(@sorted);
	}
	my $cnt = 1;
	for(my $i = 0; $i < $run && $i <scalar(@sorted); $i++){
	    if(!defined $sorted[$i]->{psfile}){ #in single-sequence-case
		if(!defined $table->{$sorted[$i]->{rRNA}}->{$sorted[$i]->{u}}){ #no duplicate
		    $table->{$sorted[$i]->{rRNA}}->{$sorted[$i]->{u}} = 1;
		    push(@{$best}, $sorted[$i]);
		    print PRED  $cnt." | ".$sorted[$i]->{snoRNA}." | ".$sorted[$i]->{rRNA}."-".$sorted[$i]->{u}." | ".$sorted[$i]->{tarSeq}." | ".$sorted[$i]->{energy}." | ".(1-$sorted[$i]->{score})."\n".$sorted[$i]->{struc}."\n".$sorted[$i]->{rawSeq}."\n";
		    my $file = `ls $dir/sno_$sorted[$i]->{header}_u_*`;
		    $file =~ s/\n//g;
		    $file =~ s/$dir/$dir\/ps_files/g;
		    my $single; #for output in table
		    $single->{type} = "svm";
		    $single->{snoRNA} = "\n".$sorted[$i]->{snoRNA};
		    $single->{mod} = $sorted[$i]->{rRNA}."-".$sorted[$i]->{u};
		    $single->{score} = 1-$sorted[$i]->{score};
		    $single->{energy} = $sorted[$i]->{energy};
		    $single->{tarSeq} = $sorted[$i]->{tarSeq};
		    $single->{struc} = $sorted[$i]->{struc};
		    $single->{out} = $dir."/".$dir.".target";
		    $single->{file} = $file;
		    push @{$out},$single;
		    $cnt++;
		}
		else{
		    $run++;
		}
	    }
	    else{ #for alignments
		if(!defined $table->{$sorted[$i]->{rRNA}}->{$sorted[$i]->{aliU}} && $sorted[$i]->{uAlign}==$sorted[$i]->{aliU}){
		    $table->{$sorted[$i]->{rRNA}}->{$sorted[$i]->{aliU}} = 1;
		    push(@{$best}, $sorted[$i]);
		    print PRED $cnt." | ".$sorted[$i]->{snoRNA}." | ".$sorted[$i]->{rRNA}."-".$sorted[$i]->{uAlign}."| ".$sorted[$i]->{rRNA}."_dme-".$sorted[$i]->{u}." | ".$sorted[$i]->{energy}." |noSeq: ".$sorted[$i]->{noSeq}." | ".$sorted[$i]->{psfile}."\n".$sorted[$i]->{struc}."\n".$sorted[$i]->{rawSeq}."\n";
		    my $align;
		    $align->{type} = "align";
		    $align->{snoRNA} = $sorted[$i]->{snoRNA};
		    $align->{mod} = $sorted[$i]->{rRNA}."-".$sorted[$i]->{uAlign};
		    $align->{noSeq} = $sorted[$i]->{noSeq};
		    $align->{energy} = $sorted[$i]->{energy};
		    if($sorted[$i]->{psfile} =~ /^No/){
			$align->{sno_ps} = "No ps-file was produced";
			$align->{str_ps} = "No ps-file was produced";
		    }
		    else{
			$align->{sno_ps} = $dir."/ps_files/sno".$sorted[$i]->{psfile};
			$align->{str_ps} = $dir."/ps_files/str".$sorted[$i]->{psfile};
		    }
		    $align->{singleU} =  " ".$sorted[$i]->{rRNA}."-".$sorted[$i]->{u};
		    push @{$out},$align;
		    $cnt++;
		}
		else{
		    $run++;
		}
		if($cnt == 0){
		    my $align;
		    $align->{type} = "align";
		    $align->{singleU} =  " ".$sorted[$i]->{rRNA}.$sorted[$i]->{u};
		    $align->{err}= "kein Target fuer das U\n";
		    push @{$out},$align;
		}
	    }
	}
    }
    else{
	print "kein Target fuer das U\n";
    }
    print PRED "\n";
    return $best;
}
############################################################################
sub cutAlign
#the parts around r target candidates are taken from alignment to perform RNAsnoop -A
############################################################################
{
    my $results = shift;
    my $seq;my $start;my $bindLen;
    my $uSpez;
    my $master;
    my $aliSnoop = '';

    my $out = Bio::AlignIO ->new (-file => ">$dir/stem", -format =>'fasta'); #write alignment in fasta format for mlocarna

    $out->write_aln($stemAlign);

    my $badsno = badLoc(); #delivers bad conserved sequences in the snoRNA
    my $in = Bio::AlignIO ->new (-file => "<$dir/mLocStem.aln", -format =>'clustalw'); #danach nochmal einlesen, um es zu bereinigen		
    my $stemAlign = $in -> next_aln();

    for(my $i = 0; $i<= $#$results ;$i++){		#durchlaufen der moeglichen hits
	print "checking conservation of target around ".$results->[$i]->{rRNA}."-".$results->[$i]->{u}."\n"; 
#	print $results->[$i]->{rRNA}."\t".$results->[$i]->{snoRNA}."\t".$results->[$i]->{u}."\t".$results->[$i]->{uAlign}."\n";

#	exit(0);
	my $cutAlign = new Bio::SimpleAlign();
	my $id = $results->[$i]->{u};
	foreach my $spezStem ($stemAlign->each_seq()){  #hier nehm ich jede Sequenz aus dem StemAlignment
	    my $nameSno = $spezStem ->display_name; #name of sequence in snoRNA-alignment
	    $nameSno =~ /^([^_]+)(_)*/;
	    my $spez = $1; #hopefully a name that makes sense and exist in both alignments
	    #exit(0);
	    foreach my $spezTarg($results->[$i]->{Talign}->each_seq()){ 	#jetzt wird im zurecht geschnittenen TargetAlignment 
		if($spezTarg->display_name =~ /$spez/){			#nach einer Sequenz mit dem selben Namen gesucht
		    my $nameTar = $spezTarg->display_name;              #name of the sequence in the target-alignment
		    $seq = $spezTarg->seq();			#Sequenz extrahieren
#		    $seq =~ s/-//g;					#gaps entfernen 
		    $bindLen = $results ->[$i]->{Tlength};	#Laenge der Interaktion im preselOrganismus, von snoop berechnet
		    my $seqStart = $spezTarg->start();		#das braucht Bioperl, ne Anfangs und Startposition fuer ne LocatableSequence
		    my $seqEnd= $seqStart+$bindLen;
		    my $vorU = ($results->[$i]->{Tleft});
		    if(length $seq < $bindLen){	#wenn es gaps in dem tarAlig gibt, d.h. nach entfernen der Gaps ist sie kuerzer dann neu ausschneiden
			foreach my $gapTarg($tarAlign->each_seq_with_id($nameTar)){	#dann suche ich zu der Spezies nochmal die ganze Seq
			    if(defined ($gapTarg -> location_from_column($results->[$i]->{uAlign}))){
				$uSpez = $gapTarg -> location_from_column($results->[$i]->{uAlign})->to_FTstring;  #pos von U in Spez finden
				if($uSpez =~ /(\d+)\^(\d+)/){ #ne Gap an der U-Stelle, darum Location mit 2 Zahlen
				    $uSpez = $1;		#nehm ich halt die kleinere Position
				}
				else{
				    $seq = $gapTarg->seq(); 
#				    $seq =~ s/-//g;		#damit ich aus der rohen Sequenz den Bereich um USpez auschneiden kann
				    $seqStart = $uSpez - $vorU;
				    $seq = substr($seq,($seqStart-1),$bindLen); #ausschneiden der Sequenz der entsprechenden Laenge
				}
			    }
			    else{		#hier gibt die location_Fkt nix zurueck, weil U noch vor Beginn der sequenz sein muesste
				$seq = 'xx'; #so wird sie bei add_seq nachher einfach uebergangen
				if($gapTarg->display_name eq "dm3"){print "PLOET";exit(0);}
			    }
			}
		    }
		    
		    #Sequenzen zu neuem Alignment packen
		    if(length($seq) >= $bindLen){
			my $tarSeq = Bio::LocatableSeq -> new(-seq => $seq, -id => $spez, -start => $seqStart, -end => $seqEnd); #here I change the name
			$cutAlign->add_seq($tarSeq);
		    }
		}
	    }
	}
	$out = Bio::AlignIO ->new (-file => ">$dir/newTar", -format =>'fasta'); 	#Alignment for snoop schreiben
	$out -> write_aln($cutAlign);
	my $noSeq = $cutAlign->no_sequences;
	if($noSeq >= $opts{N}){
	    #	if($results->[$i]->{u} == 109){exit(0);};
	    my $clustal = `clustalw $dir/newTar -outorder=input`;  #nochmal neu alignen, in richtiger Reihenfolge
	    $in = Bio::AlignIO ->new (-file => "<$dir/newTar.aln", -format =>'clustalw'); #danach nochmal einlesen, um es zu bereinigen		
	    my $tmp = $in -> next_aln();
	    my $badpos = badScore($clustal,$tmp);			#schlecht alignte Sequenzen raus, Array mit Pos der schlechten Sequenz
	    my $bad = badU($results->[$i]->{u},$tmp);		#das U ist nicht konserviert, Array mit Namen der schlechten Sequenz
	    foreach(@{$badpos}){		#numberToName 
		my $badSpez = $tmp->get_seq_by_pos($_)->display_name;
		if($badSpez ne $opts{O}){
		    push(@{$bad},$badSpez);     #Duplikate koennen enthalten sein
		}
	    }
#	    foreach(@{$bad}){
#		my $this = $_;
#		print "RAUS: ".$this."\t";
#	    }
	    my $newbad;
	    @{$newbad} = (@{$bad},@{$badsno}); #join the arrays of bad targets and badStem sequences
	    print $id."\n";
#	    foreach (@{$newbad}){
#		print" MEHR: ".$_."\t";
#	    }
	    $noSeq = cleanAligns($newbad,$tmp);   #schlechte Sequenzen rauswerfen und Alignments fuer snoop schreiben
#	    print $noSeq."\n";
	    #	exit(0);
	    if($noSeq>=$opts{N}){
		#das neue Alignment pack ich zusammen mit dem Stemalignment in snoop
		my $thisAliSnoop .= `RNAsnoop -A -t $dir/newnewTar.aln -s $dir/newStem.aln  -N`;
		`cp $dir/newnewTar.aln  $dir/$id.tar.aln`;
		`cp $dir/newStem.aln  $dir/$id.stem.aln`;
#		    print $thisAliSnoop."\n";
		if(length($thisAliSnoop)>0 && $thisAliSnoop =~ /^[^no target]/){
		    $aliSnoop .= ">".$results->[$i]->{snoRNA}."\n>".$results->[$i]->{rRNA}."-".$results->[$i]->{uAlign}."-".$results->[$i]->{startAlign}."-".$noSeq."-".$results->[$i]->{u}."\n".$thisAliSnoop;
		}
	    }
	    else{
		`cp $dir/newnewTar.aln  $dir/$id.tar.aln`;
		`cp $dir/newStem.aln  $dir/$id.stem.aln`;
		print "fewer than ".$opts{N}." sequences left, nothing is done here\n";
	    }
	}
    }
    print "done\n";#.$aliSnoop."\n";
#    exit(0);
    my $res = parseAliSnoop($aliSnoop);
    # print "HIER".$#$res."\n";
    print PRED "###############ALIGNMENT#########################\n";
    my $best = bestSel($res);
}

######################################################################
sub parseAliSnoop
#parse RNAsnoop -A output
#####################################################################    
{
    my $aliSnoop = shift;
    my $hits;
    my $snoRNA;
    my $rRNA;
    my $u;
    my $noSeq;
    my $cnt=0;
    my $uAlign;
    my $id;
    my $startAlign;
    my @snoop = split(/\n/,$aliSnoop);
    for(my $i =0; $i < scalar(@snoop); $i+=2){
	my $hit;
	if($snoop[$i] =~ /^>/ && $snoop[$i+1] =~ /^>/){
#	    print "TEST".$snoop[$i]."\n".$snoop[$i+1]."\n";
	    $snoRNA=$snoop[$i];
	    $snoop[$i+1] =~ /^>([^\-]+)\-([^\-]+)\-([^\-]+)\-(\d+)\-(\d+)/;
	    $rRNA=$1;
	    $uAlign=$2;
	    $startAlign =$3;
	    $noSeq=$4;
	    $u=$5;
	    $cnt=0;
	}
	$hit->{snoRNA}=$snoRNA;
	$hit->{rRNA}= $rRNA;
	$hit->{uAlign}= $uAlign;
	$hit->{startAlign}=$startAlign -1;
	$hit->{noSeq} = $noSeq;
	$hit->{u} = $u;
	$id = $dir."/".$hit->{u};
#	print "WAS GEHT".$hit->{snoRNA}."\t".$hit->{rRNA}."\t".$hit->{uAlign}."\t".$hit->{startAlign}."\n";#."\t".$hit->{u}."\n";
	if($snoop[$i] =~ /^[.<]/){
	    $hit->{rawStruc} = $snoop[$i];
	    $hit->{rawSeq} = $snoop[$i+1];
	    if($snoop[$i] =~ /^([^\s]+)\s([^;]+;)\s*(\d+)([^:]+):\s*\d+\s*,\s*\d+\s*\(([^=]+)=([^;]+);([^=]+)=\s*([^;]+);([^=]+)=\s*([^\)]+)\)/){
		$hit->{struc}=$1;
		$hit->{energy} = $5;
		$hit->{score} = $5;
		$hit->{uSnoop} = $3;
#		print $hit->{uSnoop}."\t";
		$hit->{aliU} = ($hit->{uSnoop}+$hit->{startAlign});
#		print $hit->{aliU}."\n";
#		print $id."\n";
#	    print $hit->{uSnoop}."\n";
#		print "U".$hit->{u}."\t".$hit->{uSnoop}."\t".$hit->{uAlign}."\n";
#       format of the psfilename: snoaln_0_u_9_dm3-1846-1861_dm3_chrX-ready-1-58.ps;straln_0_u_9_dm3-1846-1861_dm3_chrX-ready-1-58.ps get it with grep,sed awk
		$hit->{psfile} = "aln_".$cnt."_u_".$hit->{uSnoop}."_".`grep -m1 \/ $id.tar.aln |awk {'print \$1'}|sed "s/\\//-/g"|sed "s/\\n//g"`."_".`grep -m1 \/ $id.stem.aln |awk {'print \$1'}|sed "s/\\//-/g"|sed "s/\\n//g"`.".ps";
		$hit->{psfile}=~s/\n//g;
#	$hit->{duplCov} = $5;
#	$hit->{stemCov} = $7;
#	$hit->{score} = $hit->{energy} + $hit->{duplCov} + $hit->{stemCov};   
#	print $hit->{energy}." | ".$hit->{duplCov}." | ".$hit->{stemCov}." | "."\n";
		if(defined($snoop[$i+2]) && $snoop[$i+2]=~/No/){
		    $hit->{psfile}="No ps-file was produced";
		    $cnt--;
		}
#	print $hit->{score}."\n";
		push(@{$hits},$hit);
		$cnt++;
	    }
	}
	if($snoop[$i]=~/No/){
	    $i--;
	    $cnt++;
	}
    }
#   print "BLA".$#$hits."\n";
    return $hits;
}
#########################################################################
sub badScore
#returns an array with positions bad aligned targetSequences in Alignment
#bad aligned is defined by clustalw-score
#########################################################################
{
	my $clustal = shift;    #der output den clustal beim neu alignen zurueckgibt, da stehen die scores fuer die Sequenzen drin
	my $align = shift;      #die neu alignte Targetregion
	my @scores = split(/\n/,$clustal);
	my %seqScore ;
	my $noSeq = $align->no_sequences;
	for(my $j = 1; $j <= $noSeq; $j++){   #initialisierung alles Sequenzen mit 0, start bei 1, da BioPerl erst bei 1 anfaengt zu zaehlen
		$seqScore{$j}=0;
	}
	for(my $i = 0; $i < $#scores; $i++){  
		if($scores[$i] =~ /Sequences\s*\((\d+):(\d+)\)\s*Aligned.\s*Score:\s*(\d+)/){  #die interessanten Zeilen des Clustaloutputs
			$seqScore{$1}+=$3;   #zu jeder Sequenz werden alle pairwise scores addiert
			$seqScore{$2}+=$3;
		}
	}
	my @bad;
	foreach my $seq(keys(%seqScore)){
#	    print $seqScore{$seq}/$noSeq."\n";
		if($seqScore{$seq}/$noSeq < 50 ){  #wenn der score nach normalisierung zu schlecht ist
			push @bad,$seq;		   #merke ich mir die Sequenznummer
		}
	}
	return \@bad;
}
###########################################################################
sub badU
#returns sequence which haven't got an U at the position which is predicted to be pseudouridylated 
###########################################################################
{	#pruefen ob U aligniert ist 
	my $uPreSel = shift;	#pos von U im preSelOrg
	my $align = shift;	#der ausgeschnippelte Target-teil
	my $uAli = findAlignPos($align,$uPreSel,$opts{O});
	my @bad;
	foreach my $seq($align->each_seq){	#zu jeder Sequenz im neuen Targetalignment pruefe ich die konservierung des Us
		my $spez = $seq->display_name;
		if($seq->subseq($uAli,$uAli) ne "U"){   #da ist kein U, wo eins sein sollte
#		    print $spez."\n";
			push @bad, $spez;
		}
	}
	return\@bad;
}
#############################################################################
sub badLoc
##returns an array with positions bad aligned targetSequences in Alignment
#bad aligned is defined by mlocarna-score
#############################################################################
{
    my $mloc = `mlocarna $dir/stem`;
    my @scores = split(/\n/,$mloc);
    my %seqScores;
    my $noSeq = $stemAlign->no_sequences;
    my @bad;
    
    foreach(@scores){
	if($_ =~ /Score/){
	    my $this = $_;
	    $this =~ /^([^_]+)_[^\s]+\sand\s([^_]+)_[^\s]+\sScore:\s([-\d]+)/;
#	    print $this."\n";
	    my $spez1= $1;
	    my $spez2 = $2;
	    my $score= $3;
	    if(!defined $seqScores{$spez1}){$seqScores{$spez1} = 0;} #initialize
	    elsif(!defined $seqScores{$spez2}){$seqScores{$spez2} = 0;}
	    $seqScores{$spez1} += $score; #add score for sequences
	    $seqScores{$spez2} += $score; 
	}
    }
    foreach my $spez(keys(%seqScores)){
	if($seqScores{$spez}/$noSeq < 2500){
	    push @bad,$spez;
	}
#	print $spez."\t".$seqScores{$spez}/$noSeq."\n";
    }
    `mv $dir/stem.out/results/result.aln $dir/mLocStem.aln`;
    `rm -r $dir/stem.out`; #delete output of mlocarna, because we do not need it
    return \@bad;   
}
##############################################################################
sub cleanAligns
#removes the provided sequences from the alignments
##############################################################################
{
	my $bad = shift;
	my $newTar = shift;  #das 'dreckige' zusammengeschnippelte TargetAlignment
	my $newStem = new Bio::SimpleAlign();
	my @unique =&delDupli(@{$bad});

#	print scalar(@unique)."\n";
#	exit(0);

	foreach my $seq($newTar->each_seq){	
		my $spez = $seq->display_name;
		my $flag = 0;
		foreach(@unique){
			if($_ =~ /$spez/){
				$newTar->remove_seq($seq);
				$flag = 1;
			}
		}	
		 #such die Sequenz im stemAlignment und schreib sie ins neues Alignment
		if($flag == 0){
		    foreach my $seq($stemAlign->each_seq){
			if($seq->display_name =~ /$spez/){
	 			$newStem -> add_seq($seq);
			}
		    }
		}
	}
	my $noSeq = $newTar->no_sequences;
#	$newStem=$newStem->remove_columns('match');
#	$newStem=$newStem->remove_gaps('-',1);
	my $out = Bio::AlignIO-> new (-file => ">$dir/newStem.aln", -format => 'clustalw');
	foreach my $seq ($newStem->each_seq){
	  if($seq->display_name=~/$opts{O}/) {
	    $newStem->remove_seq($seq);
	    $newStem->add_seq(-SEQ=>$seq, -ORDER=>1);
	  }
	}
	$out -> write_aln($newStem);
	$out = Bio::AlignIO-> new(-file =>">$dir/newnewTar.aln", -format => 'clustalw');
	foreach my $seq ($newTar->each_seq){
	  if($seq->display_name=~/$opts{O}/) {
	    $newTar->remove_seq($seq);
	    $newTar->add_seq(-SEQ=>$seq, -ORDER=>1);
	  }
	}
	$out-> write_aln($newTar);
	
#	}
	return $noSeq;
}
sub mktable{

    my $table = shift;
    for(my $i=0;$i < $#$table && $i<=$opts{R};$i++){
	if($table->[$i]->{type} eq "svm"){
#	    print "1".$table->[$i]->{mod}."\n";
	    my $thismod = $table->[$i]->{mod};
	    printf TAB "%-15s\t%-15s\t%-10s\t%-8s\t%-20s\t%-105s\t%-45s\t%-85s\t",$table->[$i]->{snoRNA},$table->[$i]->{mod},$table->[$i]->{score},$table->[$i]->{energy},$table->[$i]->{tarSeq},$table->[$i]->{struc},$table->[$i]->{out},$table->[$i]->{file};
	    for(my $j =0; $j+$i <= $#$table; $j++){
		if($table->[$i+$j]->{type} eq "align"){
#		    print "2".$table->[$i+$j]->{singleU}." ".$thismod."\t".$table->[$i+$j]->{type}."\n";
		    if($table->[$i+$j]->{singleU} eq " ".$thismod){
			if(defined($table->[$i+$j]->{err})){
			    print "putative target is not conserved";
			    print TAB $table->[$i+$j]->{err};
			}
			else{
			    print "target was predicted at ".$table->[$i+$j]->{singleU}."\n";
			    printf TAB "%-12s\t%-18s\t%-6s\t%-85s\t%-85s",$table->[$i+$j]->{mod},$table->[$i+$j]->{noSeq},$table->[$i+$j]->{energy},$table->[$i+$j]->{sno_ps},$table->[$i+$j]->{str_ps};
			}
		    }
		}
	    }
	}
    }
}

sub findAlignPos{
	my $align = shift;
	my $pos = shift;
	my $rRNA = shift;

	my $alnpos = $align -> column_from_residue_number($rRNA, $pos);
	
	return $alnpos;
}

sub delDupli{
	my %all;
	grep {$all{$_}=0} @_;
	return (keys %all);
}

sub readfile{
	my $file = shift;
	open(FILE,"<$file") or die print $file."does not exist\n";
	my @data = <FILE>;
	print @data;
	close (FILE);
	return \@data;
}

sub printfile{
	my $file = shift;
	my $print = shift;
	open(FILE,">$file") or die print $file."does not exist\n";
	print FILE $print;
	close (FILE);
}

sub usage{
	print "sequencenames in stem-Alignment must at least be contained in sequence names of target-alignment\n";
	print "-S : SnoRNA-Alignment file only one hairpin per alignment in clustal-format\n";
	print "-T : directory with Target-Alignment file(s) in clustal-format ending with .aln \n";
	print "-O : organism for preSelection by RNAsnoop,(homo by default)\n";
	print "[-N] : minimal number of species accepted in Alignment (default 2)\n"; 
	print "[-U |-P] : accessibilities computed by RNAup or RNAplfold\n";
	print "[-s] : suffix of the RNAupfile(default u4_to_27.out \n";
	print "[-R] : how many hits should be considered(default 1)\n";
	print "[-s] : svm-scale \n";
	print "[-r] : svm-range\n";
	print "[-a] : alternative to sort hits by energy (instead of svm-score)\n";
}

